@@ -1,11 +1,15 @@
 import asyncio
 import inspect
+import os
+from pathlib import Path
 from collections import defaultdict
 
 import nest_asyncio
 import pygame
-from miniworldmaker.board_positions import board_position
-
+from miniworldmaker.board_positions import board_position, board_position_factory
+from miniworldmaker.exceptions.miniworldmaker_exception import ColorException
+from miniworldmaker.app.file_manager import FileManager
+from miniworldmaker.tools import binding
 
 class MetaAppearance(type):
     def __call__(cls, *args, **kwargs):
@@ -28,15 +32,18 @@ class Appearance(metaclass=MetaAppearance):
     """
 
     _images_dict = {}  # dict with key: image_path, value: loaded image
+    counter = 0
 
     def __init__(self):
         self.initialized = False
         self.dirty = 0
         self.blit_images = []  # Images which are blitted on the background
         self.parent = None
+        self.board = None
         self.images_list = []  # Original images
         self._image_index = 0  # current_image index (for animations)
-
+        self.id = Appearance.counter + 1
+        Appearance.counter += 1
         self.image_paths = []  # list with all images
         # properties
         self.raw_image = pygame.Surface((1, 1))  # size set in image()-method
@@ -45,6 +52,7 @@ class Appearance(metaclass=MetaAppearance):
         self.call_image_actions = {}
         self.animation_speed = 100  #: The animation speed for animations
         self._is_animated = False
+        self.loop = False
         self._is_flipped = False
         self._current_animation_images = None
         self.current_animation = None
@@ -59,7 +67,6 @@ class Appearance(metaclass=MetaAppearance):
         self._text = ""
         self._coloring = None  # Color for colorize operation
         self.draw_shapes = []
-        self._end_animation = False
         # "Action name", image_action_method, "Attribute", enabled)
         self.image_actions_pipeline = [
             ("orientation", self.image_action_set_orientation, "orientation", False),
@@ -87,44 +94,14 @@ class Appearance(metaclass=MetaAppearance):
         self.cached_images = defaultdict()
         self.animation_length = 0
 
-
-    @staticmethod
-    def load_image(path):
-        """
-        Loads an image from an path.
-
-        Args:
-            path: The path to image
-
-        Returns: The image loaded
-
-        """
-        try:
-            import os
-            canonicalized_path = os.path.join(os.path.curdir, path)
-            canonicalized_path = str(path).replace('/', os.sep).replace('\\', os.sep)
-            if canonicalized_path in Appearance._images_dict.keys():
-                # load image from img_dict
-                _image = Appearance._images_dict[canonicalized_path]
-            else:
-                try:
-                    _image = pygame.image.load(canonicalized_path).convert_alpha()
-                    Appearance._images_dict[canonicalized_path] = _image
-                except pygame.error:
-                    raise FileExistsError("File '{0}' does not exist. Check your path to the image.".format(path))
-            return _image
-        except FileExistsError:
-            raise FileExistsError("File '{0}' does not exist. Check your path to the image.".format(path))
-
     def after_init(self):
         self._reload_all()
         # self.update()
         self.initialized = True
 
     def _reload_all(self):
-        for img_action in self.image_actions_pipeline:
-            self.reload_actions[img_action[0]] = True
         self.dirty = 1
+        self.call_action("all")
 
     @property
     def font_size(self):
@@ -167,7 +144,7 @@ class Appearance(metaclass=MetaAppearance):
 
         Examples:
 
-            >>> class Player(miniworldmaker.Token):
+            >>> class Player(Token):
             >>>    def on_setup(self):
             >>>         self.add_image("background_image.jpg")
             >>>         self.costume.is_upscaled = True
@@ -186,7 +163,7 @@ class Appearance(metaclass=MetaAppearance):
 
         Examples:
 
-            >>> class Player(miniworldmaker.Token):
+            >>> class Player(Token):
             >>>    def on_setup(self):
             >>>         self.add_image("background_image.jpg")
             >>>         self.costume.is_rotatable = True
@@ -205,7 +182,7 @@ class Appearance(metaclass=MetaAppearance):
 
         Examples:
 
-            >>> class Player(miniworldmaker.Token):
+            >>> class Player(Token):
             >>>    def on_setup(self):
             >>>         self.add_image("background_image.jpg")
             >>>         self.costume.flip_vertical = True
@@ -213,9 +190,6 @@ class Appearance(metaclass=MetaAppearance):
         """
         return self._flip_vertical
 
-    def set_parent_attributes(self, attribute):
-        pass
-
     @flip_vertical.setter
     def flip_vertical(self, value):
         self._flip_vertical = value
@@ -229,7 +203,7 @@ class Appearance(metaclass=MetaAppearance):
 
         Examples:
 
-            >>> class Player(miniworldmaker.Token):
+            >>> class Player(Token):
             >>>    def on_setup(self):
             >>>         self.add_image("background_image.jpg")
             >>>         self.costume.orientation = -90
@@ -239,8 +213,6 @@ class Appearance(metaclass=MetaAppearance):
     @orientation.setter
     def orientation(self, value):
         self._orientation = value
-        if self.orientation != self.parent._orientation:
-            self.parent.orientation = self._orientation
         self.call_action("orientation")
 
     @property
@@ -315,13 +287,11 @@ class Appearance(metaclass=MetaAppearance):
     def text(self, value):
         if value == "":
             self._text = ""
-            # self.surface_loaded = False
             self.dirty = 1
         else:
             self._text = value
-            # self.surface_loaded = False
             self.dirty = 1
-        self.call_action("write_text")
+        # self.call_action("write_text") #@todo: Write text should be sufficient
         self._reload_all()
 
     def fill(self, color):
@@ -331,7 +301,7 @@ class Appearance(metaclass=MetaAppearance):
             self.dirty = 1
         except TypeError:
             self.parent.window.log("ERROR: color should be a 4-tuple (r, g, b, alpha)")
-            raise ()
+            raise ColorException("ERROR: color should be a 4-tuple (r, g, b, alpha)")
 
     def draw_shape_append(self, shape, arguments):
         self.draw_shapes.append((shape, arguments))
@@ -346,6 +316,48 @@ class Appearance(metaclass=MetaAppearance):
         self.set_image(-1)
         self._reload_all()
 
+    def find_image_file(self, path: str) -> str:
+        """
+        Search if file exists.
+        Corrects path if file ending is missing.
+        """
+        canonicalized_path = FileManager.relative_to_absolute_path(path)
+        if Path(canonicalized_path).is_file():
+            return Path(canonicalized_path)
+        else:
+            if not FileManager.has_ending:
+                return FileManager.get_path_with_file_ending(path, ["jpg", "jpeg", "png"])
+            else:
+                return FileManager.get_path_with_file_ending(canonicalized_path.split(".")[0], ["jpg", "jpeg", "png"])
+
+    @staticmethod
+    def load_image(path):
+        """
+        Loads an image from an path.
+
+        Args:
+            path: The path to image
+
+        Returns: The image loaded
+
+        """
+        try:
+            canonicalized_path = str(path).replace('/', os.sep).replace('\\', os.sep)
+            if canonicalized_path in Appearance._images_dict.keys():
+                # load image from img_dict
+                _image = Appearance._images_dict[canonicalized_path]
+            else:
+                try:
+                    _image = pygame.image.load(canonicalized_path).convert_alpha()
+                    Appearance._images_dict[canonicalized_path] = _image
+                except pygame.error:
+                    raise FileExistsError(
+                        "File '{0}' does not exist. Check your path to the image.".format(path))
+            return _image
+        except FileExistsError:
+            raise FileExistsError(
+                "File '{0}' does not exist. Check your path to the image.".format(path))
+
     def add_image(self, path: str) -> int:
         """
         Adds an image to the appearance
@@ -357,12 +369,13 @@ class Appearance(metaclass=MetaAppearance):
         Returns: The index of the added image.
 
         """
+        path = self.find_image_file(path)
+        # set image by path
         _image = Appearance.load_image(path)
         self.images_list.append(_image)
         self.image_paths.append(path)
         self.dirty = 1
         self._reload_all()
-        # self.update()
         return len(self.images_list) - 1
 
     def add_images(self, list_of_paths: list):
@@ -393,41 +406,58 @@ class Appearance(metaclass=MetaAppearance):
 
     def load_surface(self) -> pygame.Surface:
         if not self.surface_loaded:
-            image = pygame.Surface(self.parent.size, pygame.SRCALPHA)
+            image = pygame.Surface((self.parent.width, self.parent.height), pygame.SRCALPHA)
             image.fill(self.fill_color)
             image.set_alpha(255)
             self.dirty = 1
             self.raw_image = image
         return self.raw_image
 
-    def reload_image(self):
-        """ Performs all actions in image pipeline"""
-        if self.dirty == 1:
-            if self._current_animation_images:
-                self._image_index = len(self.images_list) - 1
-            if self.images_list and self._image_index < len(self.images_list) and self.images_list[self._image_index]:
-                image = self.images_list[self._image_index]  # if there is a image list load image by index
-            else:  # no image files - Render raw surface
-                image = self.load_surface()
-            for img_action in self.image_actions_pipeline:
+    def _reset_image_index(self):
+        if self._current_animation_images:
+            self._image_index = len(self.images_list) - 1
+
+
+    def _load_image_by_image_index(self):
+        if self.images_list and self._image_index < len(self.images_list) and self.images_list[self._image_index]:
+            # if there is a image list load image by index
+            image = self.images_list[self._image_index]
+        else:  # no image files - Render raw surface
+            image = self.load_surface()
+        return image
+
+    def _perform_action_pipeline_on_image(self, image):
+        for img_action in self.image_actions_pipeline:
                 # If an image action is to be executed again,
                 # load the last cached image from the pipeline and execute
                 # all subsequent image actions.
                 if self.reload_actions[img_action[0]] is False \
                         and img_action[0] in self.cached_images.keys() \
                         and self.cached_images[img_action[0]]:
-                    if getattr(self, img_action[2]):
-                        if self.parent.size != (0, 0):
-                            image = self.cached_images[img_action[0]] # Reload image from cache
-                else: # reload_actions is true
-                    if getattr(self, img_action[2]):
-                        if self.parent.size != (0, 0):
-                            image = img_action[1](image, parent=self.parent) # perform image action
+                    if getattr(self, img_action[2]) and self.parent.size != (0, 0):
+                            image = self.cached_images[img_action[0]]  # Reload image from cache
+                else:  # reload_actions is true
+                    if getattr(self, img_action[2]) and self.parent.size != (0, 0):
+                            # perform image action
+                            image = img_action[1](image, parent=self.parent)
                             self.cached_images[img_action[0]] = image
                     self.parent.dirty = 1
+        return image
 
-            for blit_image in self.blit_images:
+    def _blit_on_image(self, image):
+        for blit_image in self.blit_images:
                 image.blit(blit_image[0], blit_image[1])
+
+        return image
+
+
+    def reload_image(self):
+        """ Performs all actions in image pipeline"""
+        if self.dirty == 1:
+            self._reset_image_index()
+            image = self._load_image_by_image_index()
+            image = self._perform_action_pipeline_on_image(image)
+            image = self._blit_on_image(image)
             self._image = image
             self.dirty = 0
             for key in self.reload_actions.keys():
@@ -442,6 +472,9 @@ class Appearance(metaclass=MetaAppearance):
             if self._image_index < len(self.images_list) - 1:
                 self._image_index = self._image_index + 1
             else:
+                if not self.loop:
+                    self.is_animated = False
+                    self.after_animation()
                 self._image_index = 0
             self.dirty = 1
             self.parent.dirty = 1
@@ -456,11 +489,11 @@ class Appearance(metaclass=MetaAppearance):
         self.parent.dirty = 1
         self._reload_all()
 
-    def set_image(self, value : int) -> bool:
+    def set_image(self, value: int) -> bool:
         if value == -1:
             value = len(self.images_list) - 1
         if 0 <= value < len(self.images_list):
-            self.image_index = value
+            self._image_index = value
             self.dirty = 1
             self.parent.dirty = 1
             self._reload_all()
@@ -476,6 +509,9 @@ class Appearance(metaclass=MetaAppearance):
         return 1
 
     async def _update(self):
+        """ 
+        method is overwritten in subclasses costume and background.
+        """
         pass
 
     @property
@@ -485,7 +521,7 @@ class Appearance(metaclass=MetaAppearance):
 
         Examples:
 
-            >>> class Robot(Actor):
+            >>> class Robot(Token):
             >>>
             >>> def __init__(self, position):
             >>>     super().__init__(position)
@@ -519,7 +555,7 @@ class Appearance(metaclass=MetaAppearance):
                                           search_color=color,
                                           threshold=threshold)
 
-    def color_at(self, position: board_position.BoardPosition) -> tuple:
+    def get_color_from_pixel(self, position: board_position.BoardPosition) -> tuple:
         """
         Returns the color at a specific position
 
@@ -529,21 +565,20 @@ class Appearance(metaclass=MetaAppearance):
         Returns: The color
 
         """
-        if type(position) == tuple:
-            position = board_position.BoardPosition(position[0], position[1])
-        if position.is_on_board():
-            return self._image.get_at(position.to_pixel())
+        print(position)
+        position = board_position_factory.BoardPositionFactory(self).create(position)
+        return self._image.get_at(position.to_int())
 
     def call_action(self, action):
         reload = False
         for img_action in self.image_actions_pipeline:
             if img_action[0] == action or action == "all":
-                reload = True # reload image action
+                reload = True  # reload image action
             if reload:
-                self.reload_actions[img_action[0]] = True # reload all actions after image action
+                self.reload_actions[img_action[0]] = True  # reload all actions after image action
         self.dirty = 1
         self.parent.dirty = 1
-        #self.update()
+        # self.update()
 
     def call_actions(self, actions):
         reload = False
@@ -557,12 +592,7 @@ class Appearance(metaclass=MetaAppearance):
         return self.image
 
     def call_all_actions(self):
-        for img_action in self.image_actions_pipeline:
-            self.reload_actions[img_action[0]] = True
-        self.dirty = 1
-        self.parent.dirty = 1
-        #self.update()
-        return self.image
+        self.call_action("all")
 
     def image_action_draw_shapes(self, image: pygame.Surface, parent) -> pygame.Surface:
         for draw_action in self.draw_shapes:
@@ -570,7 +600,7 @@ class Appearance(metaclass=MetaAppearance):
         return image
 
     def image_action_texture(self, image, parent):
-        background = pygame.Surface(parent.size)
+        background = pygame.Surface((self.parent.width, self.parent.height))
         background.fill((255, 255, 255))
         i, j, width, height = 0, 0, 0, 0
         while width < parent.width:
@@ -585,8 +615,8 @@ class Appearance(metaclass=MetaAppearance):
 
     def image_action_upscale(self, image: pygame.Surface, parent) -> pygame.Surface:
         if parent.size != 0:
-            scale_factor_x = parent.size[0] / image.get_width()
-            scale_factor_y = parent.size[1] / image.get_height()
+            scale_factor_x = parent.width / image.get_width()
+            scale_factor_y = parent.height / image.get_height()
             scale_factor = min(scale_factor_x, scale_factor_y)
             new_width = int(image.get_width() * scale_factor)
             new_height = int(image.get_height() * scale_factor)
@@ -594,18 +624,18 @@ class Appearance(metaclass=MetaAppearance):
         return image
 
     def image_action_scale(self, image: pygame.Surface, parent, ) -> pygame.Surface:
-        image = pygame.transform.scale(image, parent.size)
+        image = pygame.transform.scale(image, (self.parent.width, self.parent.height))
         return image
 
     def image_action_scale_to_height(self, image: pygame.Surface, parent, ) -> pygame.Surface:
-        scale_factor = parent.size[1] / image.get_height()
+        scale_factor = parent.height / image.get_height()
         new_width = int(image.get_width() * scale_factor)
         new_height = int(image.get_height() * scale_factor)
         image = pygame.transform.scale(image, (new_width, new_height))
         return image
 
     def image_action_scale_to_width(self, image: pygame.Surface, parent, ) -> pygame.Surface:
-        scale_factor = parent.size[0] / image.get_width()
+        scale_factor = parent.width / image.get_width()
         new_width = int(image.get_width() * scale_factor)
         new_height = int(image.get_height() * scale_factor)
         image = pygame.transform.scale(image, (new_width, new_height))
@@ -613,7 +643,9 @@ class Appearance(metaclass=MetaAppearance):
 
     def image_action_rotate(self, image: pygame.Surface, parent) -> pygame.Surface:
         if self.parent.direction != 0:
-            return pygame.transform.rotate(image, - (self.parent.direction))
+            rotated_image = pygame.transform.rotozoom(image, - (self.parent.direction), 1)
+            #image.blit(rotated_image, new_rect)
+            return rotated_image
         else:
             return image
 
@@ -652,16 +684,16 @@ class Appearance(metaclass=MetaAppearance):
         return image
 
     @staticmethod
-    def crop_image(image: pygame.Surface, parent, appearance) -> pygame.Surface:
-        cropped_surface = pygame.Surface(parent.size)
+    def crop_image(self, image: pygame.Surface, parent, appearance) -> pygame.Surface:
+        cropped_surface = pygame.Surface((self.parent.width, self.parent.height))
         cropped_surface.fill((255, 255, 255))
-        cropped_surface.blit(image, (0, 0), (0, 0, parent.size[0], parent.size[1]))
+        cropped_surface.blit(image, (0, 0), (0, 0, (self.parent.width, self.parent.height)))
         return cropped_surface
 
     def image_action_write_text(self, image: pygame.Surface, parent) -> pygame.Surface:
         font_size = 0
         if self.font_size == 0:
-            font_size = parent.size[1]
+            font_size = parent.width
         else:
             font_size = self.font_size
         if self.font_path is None:
@@ -676,21 +708,20 @@ class Appearance(metaclass=MetaAppearance):
         self.font_path = font
         self.font_size = font_size
 
-    def animate(self, text, images):
-        self.end_animation()
-        self._current_animation_images = []
-        self.current_animation = text
-        for image in reversed(images):
-            image_index = self.add_image(image)
-            self.animation_length += 1
-            self._current_animation_images.append(self.images_list[image_index])
-            self.set_image(-1)
-            self.dirty = 1
-        self.add_image(images[0])  # workdaround as dummy
-        self.animation_length += 1
+    def animate(self):
+        self.is_animated = True
 
-    def end_animation(self):
-        self._end_animation = True
-        while (self.animation_length > 0):
-            self.update()
-        self._end_animation = False
+    def after_animation(self):
+        """
+        the method is overwritten in subclasses costume and appearance
+        """
+        pass
+
+
+    def register(self, method: callable):
+        """ 
+        Register method for decorator. Registers method to token or background.
+        """        
+        bound_method = binding.bind_method(self, method)
+        return bound_method
+        
